@@ -1,25 +1,20 @@
-import re
 import tempfile
 from pathlib import Path
 
 import pytest
-from ape.exceptions import ContractLogicError, SignatureError
-from evm_trace import CallTreeNode, CallType
+from ape.exceptions import ContractLogicError
+from ape.types import CallTreeNode
+from evm_trace import CallType
 from hexbytes import HexBytes
 
 from ape_foundry.exceptions import FoundryProviderError
-from ape_foundry.provider import FOUNDRY_CHAIN_ID, FoundryProvider
+from ape_foundry.provider import FOUNDRY_CHAIN_ID
 
 TEST_WALLET_ADDRESS = "0xD9b7fdb3FC0A0Aa3A507dCf0976bc23D49a9C7A3"
 
 
-@pytest.fixture(scope="module")
-def call_expression():
-    return re.compile(r"CALL: 0x([a-f]|[A-F]|\d)*.<0x([a-f]|[A-F]|\d)*> \[\d* gas]")
-
-
-def test_instantiation(disconnected_provider):
-    assert disconnected_provider.name == "foundry"
+def test_instantiation(disconnected_provider, name):
+    assert disconnected_provider.name == name
 
 
 def test_connect_and_disconnect(disconnected_provider):
@@ -44,29 +39,17 @@ def test_gas_price(connected_provider):
 
 
 def test_uri_disconnected(disconnected_provider):
-    with pytest.raises(FoundryProviderError) as err:
+    with pytest.raises(
+        FoundryProviderError, match=r"Can't build URI before `connect\(\)` is called\."
+    ):
         _ = disconnected_provider.uri
 
-    assert "Can't build URI before `connect()` is called." in str(err.value)
-
 
 def test_uri(connected_provider):
     expected_uri = f"http://127.0.0.1:{connected_provider.port}"
     assert expected_uri in connected_provider.uri
 
 
-@pytest.mark.parametrize(
-    "method,args,expected",
-    [
-        (FoundryProvider.get_nonce, [TEST_WALLET_ADDRESS], 0),
-        (FoundryProvider.get_balance, [TEST_WALLET_ADDRESS], 0),
-        (FoundryProvider.get_code, [TEST_WALLET_ADDRESS], HexBytes("")),
-    ],
-)
-def test_rpc_methods(connected_provider, method, args, expected):
-    assert method(connected_provider, *args) == expected
-
-
 def test_set_block_gas_limit(connected_provider):
     gas_limit = connected_provider.get_block("latest").gas_limit
     assert connected_provider.set_block_gas_limit(gas_limit) is True
@@ -74,7 +57,7 @@ def test_set_block_gas_limit(connected_provider):
 
 def test_set_timestamp(connected_provider):
     start_time = connected_provider.get_block("pending").timestamp
-    connected_provider.set_timestamp(start_time + 5)  # Increase by 5 seconds
+    connected_provider.set_timestamp(start_time + 5)
     new_time = connected_provider.get_block("pending").timestamp
 
     # Adding 5 seconds but seconds can be weird so give it a 1 second margin.
@@ -93,7 +76,7 @@ def test_revert_failure(connected_provider):
 
 
 def test_get_balance(connected_provider, owner):
-    assert connected_provider.get_balance(owner)
+    assert connected_provider.get_balance(owner.address)
 
 
 def test_snapshot_and_revert(connected_provider):
@@ -121,8 +104,8 @@ def test_get_call_tree(connected_provider, sender, receiver):
     transfer = sender.transfer(receiver, 1)
     call_tree = connected_provider.get_call_tree(transfer.txn_hash)
     assert isinstance(call_tree, CallTreeNode)
-    assert call_tree.call_type == CallType.CALL
-    assert repr(call_tree) == "CALL: 0xc89D42189f0450C2b2c3c61f58Ec5d628176A1E7 [0 gas]"
+    assert call_tree.call_type == CallType.CALL.value
+    assert repr(call_tree) == "0xc89D42189f0450C2b2c3c61f58Ec5d628176A1E7.0x()"
 
 
 def test_request_timeout(connected_provider, config):
@@ -142,26 +125,18 @@ def test_send_transaction(contract_instance, owner):
     contract_instance.setNumber(10, sender=owner)
     assert contract_instance.myNumber() == 10
 
-    # Have to be in the same test because of X-dist complications
-    with pytest.raises(SignatureError):
-        contract_instance.setNumber(20)
-
 
 def test_revert(sender, contract_instance):
     # 'sender' is not the owner so it will revert (with a message)
-    with pytest.raises(ContractLogicError) as err:
+    with pytest.raises(ContractLogicError, match="!authorized"):
         contract_instance.setNumber(6, sender=sender)
 
-    assert str(err.value) == "!authorized"
-
 
 def test_contract_revert_no_message(owner, contract_instance):
     # The Contract raises empty revert when setting number to 5.
-    with pytest.raises(ContractLogicError) as err:
+    with pytest.raises(ContractLogicError, match="Transaction failed."):
         contract_instance.setNumber(5, sender=owner)
 
-    assert str(err.value) == "Transaction failed."
-
 
 def test_transaction_contract_as_sender(contract_instance, connected_provider):
     # Set balance so test wouldn't normally fail from lack of funds
